/* sm2.h
   This file is part of GNU Nettle.

   GNU Nettle is free software: you can redistribute it and/or
   modify it under the terms of either:

     * the GNU Lesser General Public License as published by the Free
       Software Foundation; either version 3 of the License, or (at your
       option) any later version.

   or

     * the GNU General Public License as published by the Free
       Software Foundation; either version 2 of the License, or (at your
       option) any later version.

   or both in parallel, as here.

   GNU Nettle is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received copies of the GNU General Public License and
   the GNU Lesser General Public License along with this program.  If
   not, see http://www.gnu.org/licenses/.
*/

#ifndef NETTLE_SM2_H_INCLUDED
# define NETTLE_SM2_H_INCLUDED

# include "sm3.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Name mangling */
#define sm2_key_prepare nettle_sm2_key_prepare
#define sm2_public_key_init nettle_sm2_public_key_init
#define sm2_private_key_init nettle_sm2_private_key_init
#define sm2_key_clear nettle_sm2_key_clear
#define sm2_sign nettle_sm2_sign
#define sm2_verify nettle_sm2_verify
#define sm2_encrypt nettle_sm2_encrypt
#define sm2_decrypt nettle_sm2_decrypt

/* SM2 */
struct sm2_ctx
{
    gcry_sexp_t pubk;           /* public-key. */
    gcry_sexp_t privk;          /* private-key. */
    gcry_sexp_t rsa_parms;      /* rsa parms. */
    gcry_sexp_t rsa_keypair;    /* rsa keypair. */
};

/* sm2 key prepare. */
int
sm2_key_prepare(struct sm2_ctx *ctx);

/* Public key initialization. */
void
sm2_public_key_init(struct sm2_ctx *ctx);

/* Private key initialization. */
void
sm2_private_key_init(struct sm2_ctx *ctx);

/* Clear key related data information. */
void
sm2_key_clear(struct sm2_ctx *ctx);

/* Private key signature. */
int
sm2_sign(const struct sm2_ctx *key,
     struct sm3_ctx *hash,
     gcry_sexp_t *signature);

/* Public key verification. */
int
sm2_verify(const struct sm2_ctx *key,
     struct sm3_ctx *hash,
     gcry_sexp_t *signature);

/* Public key encryption. */
int
sm2_encrypt(const struct sm2_ctx *key,
	    const uint8_t *src,
	    gcry_sexp_t *ciphertext);

/* Private key decryption. */
int
sm2_decrypt(const struct sm2_ctx *key,
	    const uint8_t **src,
	    gcry_sexp_t *ciphertext);


#ifdef __cplusplus
}
#endif

#endif /* NETTLE_SM3_H_INCLUDED */
