/* sm3.h
   Copyright (C) 2021 Xin Liu.
   This file is part of GNU Nettle.

   GNU Nettle is free software: you can redistribute it and/or
   modify it under the terms of either:

     * the GNU Lesser General Public License as published by the Free
       Software Foundation; either version 3 of the License, or (at your
       option) any later version.

   or

     * the GNU General Public License as published by the Free
       Software Foundation; either version 2 of the License, or (at your
       option) any later version.

   or both in parallel, as here.

   GNU Nettle is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received copies of the GNU General Public License and
   the GNU Lesser General Public License along with this program.  If
   not, see http://www.gnu.org/licenses/.
*/

#ifndef NETTLE_SM3_H_INCLUDED
#define NETTLE_SM3_H_INCLUDED

#include "nettle-types.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Name mangling */
#define sm3_init nettle_sm3_init
#define sm3_update nettle_sm3_update
#define sm3_digest nettle_sm3_digest

/* SM3 digest length. */
#define SM3_DIGEST_LENGTH 32

#include <gcrypt.h>

/* SM3 */
struct sm3_ctx
{
	int 	algo;		    /* Algorithm type. */
  gcry_md_hd_t md;
};

/* Initialize the SM3 state. */
void
sm3_init(struct sm3_ctx *ctx);

/* Hash some more data. */
void
sm3_update(struct sm3_ctx *ctx,
         size_t length,
         const uint8_t *data);

/* Performs final processing and extracts the message digest, writing it to digest.
	This function also resets the context in the same way as sm3_init. */
void
sm3_digest(struct sm3_ctx *ctx,
	       size_t length,
	       uint8_t *digest);


#ifdef __cplusplus
}
#endif

#endif /* NETTLE_SM3_H_INCLUDED */
