/* sm4.h
   Copyright (C) 2021 Xin Liu.
   This file is part of GNU Nettle.

   GNU Nettle is free software: you can redistribute it and/or
   modify it under the terms of either:

     * the GNU Lesser General Public License as published by the Free
       Software Foundation; either version 3 of the License, or (at your
       option) any later version.

   or

     * the GNU General Public License as published by the Free
       Software Foundation; either version 2 of the License, or (at your
       option) any later version.

   or both in parallel, as here.

   GNU Nettle is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received copies of the GNU General Public License and
   the GNU Lesser General Public License along with this program.  If
   not, see http://www.gnu.org/licenses/.
*/

#ifndef NETTLE_SM4_H_INCLUDED
#define NETTLE_SM4_H_INCLUDED

#include <gcrypt.h>
#include "nettle-types.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Name mangling */
#define sm4_set_key nettle_sm4_set_key
#define sm4_encrypt nettle_sm4_encrypt
#define sm4_decrypt nettle_sm4_decrypt

/* Algorithm mode. */
#define ECB GCRY_CIPHER_MODE_ECB
#define CBC GCRY_CIPHER_MODE_CBC

/* SM3 */
struct sm4_ctx
{
    gcry_cipher_hd_t cipher_hd; /* gcry_cipher_hd_t *hd. */
    uint8_t          *vector;   /* Vector value. */
    size_t           blk_len;   /* SM4 algorithm block length. */
    size_t           key_len;   /* SM4 algorithm key length. */
    int              mode;      /* Algorithm mode. */
    int              algo;      /* Algorithm type. */
};

/* Create a handle for algorithm sm4 to be used in CTX->mode,
	Set KEY for the cipher handle CTX->cipher_hd,
	Set initialization vector CTX->vector for the cipher handle CTX->cipher_hd. */
void
sm4_set_key(struct sm4_ctx *ctx,
         const uint8_t *key);

/* Use the cipher handle CTX->cipher_hd to encrypt the plaintext of size LENGTH in DST
   Enter the allocated buffer SRC of length LENGTH. for
   Most algorithms can pass NULL for DST and 0 for LENGTH
   And decrypt the data provided in SRC on-site. */
void
sm4_encrypt(const struct sm4_ctx *ctx,
		size_t length, uint8_t *dst,
		const uint8_t *src);

/* The counterpart to sm4_encrypt. */
void
sm4_decrypt(const struct sm4_ctx *ctx,
		size_t length, uint8_t *dst,
		const uint8_t *src);

#ifdef __cplusplus
}
#endif

#endif /* NETTLE_SM4_H_INCLUDED */
